[Python-checkins] [3.7] bpo-34282: Fix Enum._convert method shadowing members named _convert (GH-9034)

Ethan Furman webhook-mailer at python.org
Mon Sep 10 12:39:53 EDT 2018


https://github.com/python/cpython/commit/c0d63bf73b35df374e6e66c08b0e297fb828d744
commit: c0d63bf73b35df374e6e66c08b0e297fb828d744
branch: 3.7
author: orlnub123 <orlnub123 at gmail.com>
committer: Ethan Furman <ethan at stoneleaf.us>
date: 2018-09-10T09:39:48-07:00
summary:

[3.7] bpo-34282: Fix Enum._convert method shadowing members named _convert (GH-9034)

* Fix Enum._convert shadowing members named _convert

files:
A Misc/NEWS.d/next/Library/2018-09-02-13-33-35.bpo-34282.ztyXH8.rst
M Lib/enum.py
M Lib/test/test_enum.py

diff --git a/Lib/test/test_enum.py b/Lib/test/test_enum.py
index ef2d1daaf942..4b1722894601 100644
--- a/Lib/test/test_enum.py
+++ b/Lib/test/test_enum.py
@@ -1516,6 +1516,23 @@ class MoreColor(Color):
             yellow = 6
         self.assertEqual(MoreColor.magenta.hex(), '5 hexlified!')
 
+    def test_subclass_duplicate_name(self):
+        class Base(Enum):
+            def test(self):
+                pass
+        class Test(Base):
+            test = 1
+        self.assertIs(type(Test.test), Test)
+
+    def test_subclass_duplicate_name_dynamic(self):
+        from types import DynamicClassAttribute
+        class Base(Enum):
+            @DynamicClassAttribute
+            def test(self):
+                return 'dynamic'
+        class Test(Base):
+            test = 1
+        self.assertEqual(Test.test.test, 'dynamic')
 
     def test_no_duplicates(self):
         class UniqueEnum(Enum):
diff --git a/Misc/NEWS.d/next/Library/2018-09-02-13-33-35.bpo-34282.ztyXH8.rst b/Misc/NEWS.d/next/Library/2018-09-02-13-33-35.bpo-34282.ztyXH8.rst
new file mode 100644
index 000000000000..c1e606ab0ce5
--- /dev/null
+++ b/Misc/NEWS.d/next/Library/2018-09-02-13-33-35.bpo-34282.ztyXH8.rst
@@ -0,0 +1 @@
+Fix enum members getting shadowed by parent attributes.



More information about the Python-checkins mailing list
diff --git a/Lib/enum.py b/Lib/enum.py
index 576de031805d..69b41fe7cbc0 100644
--- a/Lib/enum.py
+++ b/Lib/enum.py
@@ -171,9 +171,11 @@ def __new__(metacls, cls, bases, classdict):
         enum_class._member_map_ = OrderedDict()      # name->value map
         enum_class._member_type_ = member_type
 
-        # save attributes from super classes so we know if we can take
-        # the shortcut of storing members in the class dict
-        base_attributes = {a for b in enum_class.mro() for a in b.__dict__}
+        # save DynamicClassAttribute attributes from super classes so we know
+        # if we can take the shortcut of storing members in the class dict
+        dynamic_attributes = {k for c in enum_class.mro()
+                              for k, v in c.__dict__.items()
+                              if isinstance(v, DynamicClassAttribute)}
 
         # Reverse value->name map for hashable values.
         enum_class._value2member_map_ = {}
@@ -233,7 +235,7 @@ def __new__(metacls, cls, bases, classdict):
                 enum_class._member_names_.append(member_name)
             # performance boost for any member that would not shadow
             # a DynamicClassAttribute
-            if member_name not in base_attributes:
+            if member_name not in dynamic_attributes:
                 setattr(enum_class, member_name, enum_member)
             # now add to _member_map_
             enum_class._member_map_[member_name] = enum_member